// Ansteuerung eines HD44780 kompatiblen LCD im 4-Bit-Interfacemodus
// http://www.mikrocontroller.net/articles/HD44780
// http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial/LCD-Ansteuerung
//
// Die Pinbelegung ist ber defines in lcd-routines.h einstellbar

//Diese Version ist fr Mega162 gedacht.
 
#include <avr/io.h>
#include "LCD-Routines.h"
#include <util/delay.h>
//#include "blink_led.h"
 
////////////////////////////////////////////////////////////////////////////////
// Erzeugt einen Enable-Puls
static void lcd_enable( void )
{
    LCD_ENABLE_PORT |= (1<<LCD_EN);     // Enable auf 1 setzen
    _delay_us( LCD_ENABLE_US );  // kurze Pause
    LCD_ENABLE_PORT &= ~(1<<LCD_EN);    // Enable auf 0 setzen
}


// Sendet eine 4-bit Ausgabeoperation an das LCD
static void lcd_out( uint8_t data )
{
    data &= 0xF0;                       // obere 4 Bit maskieren
 
    LCD_DATA_PORT &= ~(0xF0>>(4-LCD_DB));    // Maske lschen
    LCD_DATA_PORT |= (data>>(4-LCD_DB));     // Bits setzen
    lcd_enable();
}
////////////////////////////////////////////////////////////////////////////////
// Initialisierung: muss ganz am Anfang des Programms aufgerufen werden.
void lcd_init( void )
{
    // verwendete Pins auf Ausgang schalten
    uint8_t pins1 = ( (1<< PA4) |(1<< PA5) |(1<< PA6) | (1<<PA7)) ;     //LCD DB4-DB7 <-->  PORTA Bit PA4-PA7  
	uint8_t pins3 = (1<<LCD_EN);			//OK
	uint8_t pins4 = (1<<LCD_RS);			//OK

    LCD_DATA_DDR |= pins1;		//  LCD DB4-DB7 <-->  PORTA Bit PA4-PA7
	LCD_ENABLE_DDR |= pins3;	//  LCD EN      <-->  PORTC Bit PC1 
	LCD_STEUER_DDR |= pins4;	//	LCD RS 			PORTE Bit PE0
	
 
    // initial alle Ausgnge auf Null
    LCD_DATA_PORT &= ~pins1;
	LCD_ENABLE_PORT &= ~pins3;
	LCD_STEUER_PORT &= ~pins4;
 
    // warten auf die Bereitschaft des LCD
    _delay_ms( LCD_BOOTUP_MS );
    
    // Soft-Reset muss 3mal hintereinander gesendet werden zur Initialisierung
    lcd_out( LCD_SOFT_RESET );
    _delay_ms( LCD_SOFT_RESET_MS1 );
 
    lcd_enable();
    _delay_ms( LCD_SOFT_RESET_MS2 );
 
    lcd_enable();
    _delay_ms( LCD_SOFT_RESET_MS3 );
 
    // 4-bit Modus aktivieren 
    lcd_out( LCD_SET_FUNCTION |
             LCD_FUNCTION_4BIT );
    _delay_ms( LCD_SET_4BITMODE_MS );
 
    // 4-bit Modus / 2 Zeilen / 5x7
    lcd_command( LCD_SET_FUNCTION |
                 LCD_FUNCTION_4BIT |
                 LCD_FUNCTION_2LINE |
	//			LCD_FUNCTION_1LINE |
                 LCD_FUNCTION_5X7 );
 
    // Display ein / Cursor aus / Blinken aus
    lcd_command( LCD_SET_DISPLAY |
                 LCD_DISPLAY_ON |
                 LCD_CURSOR_OFF |
                 LCD_BLINKING_OFF); 
 
    // Cursor inkrement / kein Scrollen
    lcd_command( LCD_SET_ENTRY |
                 LCD_ENTRY_INCREASE |
                 LCD_ENTRY_NOSHIFT );
 
    lcd_clear();
}
  
////////////////////////////////////////////////////////////////////////////////
// Sendet ein Datenbyte an das LCD
void lcd_data( uint8_t data )
{
    LCD_STEUER_PORT |= (1<<LCD_RS);    // RS auf 1 setzen
 
    lcd_out( data );            // zuerst die oberen, 
    lcd_out( data<<4 );         // dann die unteren 4 Bit senden
 
    _delay_us( LCD_WRITEDATA_US );
}
 
////////////////////////////////////////////////////////////////////////////////
// Sendet einen Befehl an das LCD
void lcd_command( uint8_t data )
{
    LCD_STEUER_PORT &= ~(1<<LCD_RS);    // RS auf 0 setzen, -->> schreiben

    lcd_out( data );             // zuerst die oberen, 
    lcd_out( data<<4 );           // dann die unteren 4 Bit senden
 
    _delay_us( LCD_COMMAND_US );
}
 
////////////////////////////////////////////////////////////////////////////////
// Sendet den Befehl zur Lschung des Displays
void lcd_clear( void )
{
    lcd_command( LCD_CLEAR_DISPLAY );
    _delay_ms( LCD_CLEAR_DISPLAY_MS );
	
}
 
////////////////////////////////////////////////////////////////////////////////
// Sendet den Befehl: Cursor Home
void lcd_home( void )
{
    lcd_command( LCD_CURSOR_HOME );
    _delay_ms( LCD_CURSOR_HOME_MS );
}
 
////////////////////////////////////////////////////////////////////////////////
// Setzt den Cursor in Spalte x (0..15) Zeile y (1..4) 
 
void lcd_setcursor( uint8_t x, uint8_t y )
{
    uint8_t data;
 
    switch (y)
    {
        case 1:    // 1. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE1 + x;
            break;
 
        case 2:    // 2. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE2 + x;
            break;
 
        case 3:    // 3. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE3 + x;
            break;
 
        case 4:    // 4. Zeile
            data = LCD_SET_DDADR + LCD_DDADR_LINE4 + x;
            break;
 
        default:
            return;                                   // fr den Fall einer falschen Zeile
    }
 
    lcd_command( data );
}
 
////////////////////////////////////////////////////////////////////////////////
// Schreibt einen String auf das LCD
 
void lcd_string( const char *data )
{
    while( *data != '\0' )
        lcd_data( *data++ );
}
 
////////////////////////////////////////////////////////////////////////////////
// Schreibt ein Zeichen in den Character Generator RAM
/* 
void lcd_generatechar( uint8_t code, const uint8_t *data )
{
    // Startposition des Zeichens einstellen
    lcd_command( LCD_SET_CGADR | (code<<3) );
 
    // Bitmuster bertragen
    for ( uint8_t i=0; i<8; i++ )
    {
        lcd_data( data[i] );
    }
}
*/


//-------------------------------------------------------------------------------------------------
// LCD_WriteHex schreibt einByte als Hex-Zahl aufs Display
//-------------------------------------------------------------------------------------------------
inline unsigned char nibble_zu_hex_(unsigned char nib) {
 if (nib < 0x0A) nib = nib + '0';
 else nib = (nib - 0x0A) + 'A';
 return nib;
}

void LCD_WriteHex(unsigned char zahl)
{
	unsigned char temp; 
	temp = (zahl & 0x0F); 
	zahl = (zahl>>4); 
	lcd_data(nibble_zu_hex_(zahl)); //das erste Zeichen aufs LCD ausgeben
	lcd_data(nibble_zu_hex_(temp)); //das zweite Zeichen aufs LCD ausgeben
}